@@ -1,9 +1,9 @@
 package Tapper::TAP::Harness;
-BEGIN {
-  $Tapper::TAP::Harness::AUTHORITY = 'cpan:TAPPER';
-}
+# git description: v4.1.2-21-g895b848
+
+our $AUTHORITY = 'cpan:TAPPER';
 # ABSTRACT: Tapper - Tapper specific TAP handling
-$Tapper::TAP::Harness::VERSION = '4.1.2';
+$Tapper::TAP::Harness::VERSION = '5.0.0';
 use 5.010;
 use strict;
 use warnings;
@@ -61,12 +61,12 @@ use Moose;
 
 has tap            => ( is => 'rw', isa => 'Str' );
 has tap_is_archive => ( is => 'rw' );
-has parsed_report  => ( is => 'rw', isa => 'HashRef', default => sub {{}} );
+has parsed_report  => ( is => 'rw', isa => 'HashRef', default => sub {{tap_sections => []}});
 has section_names  => ( is => 'rw', isa => 'HashRef', default => sub {{}} );
 
 our $re_prove_section          = qr/^([-_\d\w\/.]*\w)\s?\.{2,}\s*$/;
 our $re_tapper_meta           = qr/^#\s*((?:Tapper|Artemis|Test)-)([-\w]+):(.+)$/i;
-our $re_tapper_meta_section   = qr/^#\s*((?:Tapper|Artemis|Test)-Section:)\s*(.+)$/i;
+our $re_tapper_meta_section   = qr/^#\s*((?:Tapper|Artemis|Test)-Section:)\s*(\S.*)$/i;
 our $re_explicit_section_start = qr/^#\s*((?:Tapper|Artemis|Test)-explicit-section-start:)\s*(\S*)/i;
 
 sub _get_prove {
@@ -79,7 +79,7 @@ sub _get_prove {
 sub _unique_section_name
 {
         my ($self, $section_name) = @_;
-
+        return if not defined $section_name;
         my $trail_number = 1;
         if (defined $self->section_names->{$section_name}
             and not $section_name =~ m/\d$/)
@@ -98,9 +98,6 @@ sub _unique_section_name
 sub _fix_broken_tap {
         my ($tap) = @_;
 
-        # say STDERR "============================================================";
-        # say STDERR $tap;
-
         # TAP::Parser chokes on that
         $tap =~ s/^(\s+---)\s+$/$1/msg;
 
@@ -122,18 +119,34 @@ sub _fix_broken_tap {
         $tap =~ s/^(\s+)(Context Switches)\s*([^\n]*)$/$1context_switches: $3/msg;
         $tap =~ s/^(\s+)(Sleeps)\s*([^\n]*)$/$1sleeps: $3/msg;
 
-        # say STDERR "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<";
-        # say STDERR $tap;
-        # say STDERR ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>";
-
         return $tap;
 }
 
+sub _parse_tap_into_sections_one_section
+{
+        my ($self) = @_;
+        my @sections = $self->_get_tap_sections_from_single;
+        $self->_collect_meta_from_sections(@sections);
+}
+
+
+sub tap_single_plan
+{
+        my ($self) = @_;
+        return if $self->tap_is_archive;
+        my @plans = ($self->tap) =~ m/(1\.\.\d+)/mg;
+        return(int(@plans) == 1);
+}
+
+
+
 sub _parse_tap_into_sections
 {
-        my ($self) = shift;
+        my ($self) = @_;
 
+        # Order matters
         return $self->_parse_tap_into_sections_archive(@_) if $self->tap_is_archive;
+        return $self->_parse_tap_into_sections_one_section(@_) if $self->tap_single_plan;
         return $self->_parse_tap_into_sections_raw(@_);
 }
 
@@ -145,8 +158,6 @@ sub _parse_tap_into_sections_raw
 {
         my ($self) = @_;
 
-        $self->parsed_report->{tap_sections} = [];
-        $self->section_names({});
 
         my $report_tap = $self->tap;
 
@@ -177,7 +188,6 @@ sub _parse_tap_into_sections_raw
                 my $is_unknown = $line->is_unknown;
                 my $is_yaml    = $line->is_yaml;
 
-                #say STDERR "__".$line->raw;
                 # prove section
                 if ( $is_unknown and $raw =~ $re_prove_section ) {
                         $looks_like_prove_output ||= 1;
@@ -187,13 +197,6 @@ sub _parse_tap_into_sections_raw
 
                 $sections_marked_explicit = 1 if $raw =~ $re_explicit_section_start;
 
-#                 say STDERR "    $raw";
-#                 say STDERR "    $i. is_version:               $is_version";
-#                 say STDERR "    $i. is_yaml:                  $is_yaml";
-#                 say STDERR "    $i. looks_like_prove_output:  $looks_like_prove_output";
-#                 say STDERR "    $i. last_line_was_plan:       $last_line_was_plan";
-#                 say STDERR "    $i. last_line_was_version:    $last_line_was_version";
-#                 say STDERR "    $i. sections_marked_explicit: $sections_marked_explicit";
 
                 # start new section
                 if ( $raw =~ $re_explicit_section_start and ! $last_line_was_version
@@ -213,9 +216,6 @@ sub _parse_tap_into_sections_raw
                               $raw =~ $re_prove_section
                             ) ) ) )
                 {
-#                         say STDERR "____________________________ new section";
-
-                        #say STDERR "****************************************";
                         if (keys %section) {
                                 # Store a copy (ie., not \%section) so it doesn't get overwritten in next loop
                                 fix_last_ok(\ $section{raw}) if $looks_like_prove_output;
@@ -240,7 +240,7 @@ sub _parse_tap_into_sections_raw
                         $val =~ s/^\s+//;
                         $val =~ s/\s+$//;
                         if ($raw =~ $re_tapper_meta_section) {
-                                $section{section_name} //= $self->_unique_section_name( $val );
+                                $section{section_name} ||= $self->_unique_section_name( $val );
                         }
                         $section{section_meta}{$key} = $val;              # section keys
                         $self->parsed_report->{report_meta}{$key} = $val; # also global keys, later entries win
@@ -287,14 +287,25 @@ sub _get_tap_sections_from_archive
         return @tap_sections;
 }
 
+sub _get_tap_sections_from_single
+{
+        my ($self) = @_;
+        my ($section_name) = ($self->tap) =~ $re_tapper_meta_section;
+        return({ tap => $self->tap, filename => $section_name}) if $section_name;
+        return({ tap => $self->tap});
+}
+
+
 sub _parse_tap_into_sections_archive
 {
         my ($self) = @_;
 
-        $self->parsed_report->{tap_sections} = [];
-        $self->section_names({});
-
         my @tap_sections = $self->_get_tap_sections_from_archive($self->tap);
+        $self->_collect_meta_from_sections(@tap_sections);
+}
+
+sub _collect_meta_from_sections {
+        my ($self, @tap_sections) = @_;
 
         my $looks_like_prove_output = 0;
         $self->parsed_report->{report_meta} = {
@@ -364,8 +375,6 @@ sub _parse_tap_into_sections_archive
         # store last section
         push @{$self->parsed_report->{tap_sections}}, { %section } if keys %section;
 
-        use Data::Dumper;
-        #print STDERR Dumper($self->parsed_report);
         $self->fix_section_names;
 }
 
@@ -402,7 +411,6 @@ sub _aggregate_sections
                 $rawtap    = $TAPVERSION."\n".$rawtap unless $rawtap =~ /^TAP Version/msi;
                 my $parser = new TAP::Parser ({ tap => $rawtap });
                 $parser->run;
-                # print STDERR "# " . $section->{section_name} . "\n";
                 $aggregator->add( $section->{section_name} => $parser );
         }
         $aggregator->stop;
@@ -476,7 +484,6 @@ sub _process_section_meta_information
         foreach my $section ( @{$self->parsed_report->{tap_sections}} ) {
                 foreach my $key (@SECTION_HEADER_KEYS_GENERAL)
                 {
-                        use Data::Dumper;
                         my $section_name = $section->{section_name};
                         my $value        = $section->{section_meta}{$key};
                         my $accessor     = $key;
@@ -499,7 +506,7 @@ sub _process_meta_information
 sub evaluate_report
 {
         my ($self) = @_;
-
+        return if @{$self->parsed_report->{tap_sections}};
         $self->_parse_tap_into_sections();
         $self->_aggregate_sections();
         $self->_process_meta_information();
@@ -531,23 +538,17 @@ sub generate_html
         my $dir        = $temp->mkdir("section");
         my $TAPVERSION = "TAP Version 13";
         my @files = map {
-                         my $fname          = "section/".$_->{section_name};
+                         if ($_->{section_name} =~ m'(^\.{1,2})|/') {
+                                 $_->{section_name} =~ s/^\./_dot_/g;
+                                 $_->{section_name} =~ s|/|_slash_|g;
+                                 $_->{section_name} = "autogenerated_filename_$_->{section_name}";
+                         }
+                         my $fname          = "section/$_->{section_name}";
                          my $rawtap         = $_->{raw};
                          $rawtap            = $TAPVERSION."\n".$rawtap unless $rawtap =~ /^TAP Version/msi;
                          my $script_content = $rawtap;
                          my $file           = $temp->touch($fname, $script_content);
 
-#                          say STDERR "--------------------------------------------------";
-#                          say STDERR $_->{raw};
-#                          say STDERR "--------------------------------------------------";
-#                          say STDERR $rawtap;
-#                          say STDERR "--------------------------------------------------";
-#                          say STDERR $script_content;
-#                          say STDERR "--------------------------------------------------";
-#                          say STDERR "$temp/$fname";
-#                          say STDERR "--------------------------------------------------";
-#                          #sleep 10;
-
                          [ "$temp/$fname" => $_->{section_name} ];
                         } @{$self->parsed_report->{tap_sections}};
 
@@ -557,7 +558,6 @@ sub generate_html
         my $prove = _get_prove();
 
         my $cmd = qq{cd $temp/section ; $^X $prove -vm --exec 'cat' --formatter=TAP::Formatter::HTML `find -type f | sed -e 's,^\./,,' | sort`};
-        #say STDERR $cmd;
         my $html = qx( $cmd );
 
         $html = _fix_generated_html( $html );
@@ -573,12 +573,16 @@ __END__
 
 =pod
 
-=encoding utf-8
+=encoding UTF-8
 
 =head1 NAME
 
 Tapper::TAP::Harness - Tapper - Tapper specific TAP handling
 
+=head2 tap_single_plan
+
+Return true when TAP contains exactly one plan
+
 =head2 fix_last_ok
 
 The C<prove> tool adds an annoying last summary line, cut that away.
@@ -598,10 +602,20 @@ aggregate the sections and extract contained meta information.
 Render TAP through TAP::Formatter::HTML and fix some formatting to fit
 into Tapper.
 
-=head1 AUTHOR
+=head1 AUTHORS
+
+=over 4
+
+=item *
 
 AMD OSRC Tapper Team <tapper@amd64.org>
 
+=item *
+
+Tapper Team <tapper-ops@amazon.com>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is Copyright (c) 2015 by Advanced Micro Devices, Inc..
